from typing import Any
from re import match


def valid_ip(ip: str) -> bool:
    """check for a valid ipV4 input

    Args:
        ip (str): ip string

    Returns:
        bool: result
    """

    res = match(
        "^(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)$",
        ip,
    )
    if res:
        return True

    else:
        return False


def cut_slash(text):

    if text[-1] == "/":
        return text[:-1]

    else:
        return text


def pick(inDict: dict, key: str, defaultRet: Any = None) -> Any:

    if isinstance(inDict, dict):
        if key in inDict.keys():
            return inDict[key]

    return defaultRet



from typing import Union, List, Dict, Literal
from dataclasses import dataclass


class ApiService:
    def __init__(
        self,
        serviceName: str,
        refUrl: str,
        apiRoute: str,
        requestMethod: Literal["GET", "POST"],
        responseType: type,
        responseContent,
        returnType: type,
        responseAlias: dict = {},
        setValue: bool = False,
        postData: dict = {},
    ) -> None:
        self.serviceName: str = serviceName
        self.refUrl: str = refUrl
        self.apiRoute: str = apiRoute
        self.requestMethod = requestMethod
        self.responseType: type = responseType
        self.responseContent: Union[str, dict, list[dict]] = responseContent
        self.returnType: type = returnType
        self.responseAlias: dict = responseAlias
        self.setValue: bool = setValue
        self.postData: dict = postData



from dataclasses import dataclass
from pydpn._api.api_base import ApiBase


@dataclass
class DpnMode:
    dpnMode: str = ""
    tunnelCode: str = ""


class Mode(ApiBase):
    @property
    def dpnMode(self) -> DpnMode:
        """get the dpn mode

        Returns:
            DpnMode: dpn Mode
        """
        desc = {
            "serviceName": "dpnMode",
            "refUrl": self.refUrl,
            "apiRoute": "smartRoute/getDpnMode",
            "requestMethod": "GET",
            "responseType": dict,
            "responseContent": {"dpnMode": str, "tunnelCode": str},
            "returnType": DpnMode,
        }

        return self.deeper._api_call(desc)

    def set_dpnMode(self, value) -> bool:
        """set the dpn mode

        Returns:
            bool: request success
        """
        if isinstance(value, str):
            if value in ["disabled", "smart"]:
                postData = {"dpnMode": value}

            else:
                raise ValueError(
                    "dpnMode must be one of [disabled, smart, full]"
                )

        if isinstance(value, DpnMode):
            value = value.__dict__()

        if isinstance(value, dict):
            keys = value.keys()
            if "dpnMode" in keys:
                dpnMode = value["dpnMode"]
                if dpnMode in ["disabled", "smart"]:
                    postData = {"dpnMode": dpnMode}

                elif dpnMode == "full":
                    if "tunnelCode" in keys:
                        postData = {
                            "dpnMode": "full",
                            "tunnelCode": value["tunnelCode"],
                        }
                else:
                    raise ValueError(
                        "dpnMode must be one of [disabled, smart, full]"
                    )

            else:
                raise ValueError("missing key dpnMode")

        desc = {
            "serviceName": "dpnMode",
            "refUrl": self.refUrl,
            "apiRoute": "smartRoute/setDpnMode",
            "requestMethod": "POST",
            "responseType": dict,
            "responseContent": {"success": bool},
            "returnType": bool,
            "setValue": True,
            "postData": postData,
        }

        return self.deeper._api_call(desc)

    @property
    def tunnelOptions(self) -> list:
        """get list of tunnel options

        Returns:
            DpnMode: dpn Mode
        """
        desc = {
            "serviceName": "dpnMode",
            "refUrl": self.refUrl,
            "apiRoute": "smartRoute/listTunnelOptions",
            "requestMethod": "GET",
            "responseType": list,
            "responseContent": {
                "regionCode": str,
                "countryCode": str,
                "tunnelCode": str,
                "numOfUse": int,
                "addedByUser": bool,
            },
            "returnType": list,
        }

        return self.deeper._api_call(desc)



from dataclasses import dataclass
from pydpn._api.api_base import ApiBase


class Tunnel(ApiBase):
    @property
    def listTunnels(self) -> list:
        """get list of tunnels

        Returns:
            DpnMode: dpn Mode
        """
        desc = {
            "serviceName": "listTunnels",
            "refUrl": self.refUrl,
            "apiRoute": "smartRoute/listTunnels",
            "requestMethod": "GET",
            "responseType": list,
            "responseContent": {
                "regionCode": str,
                "countryCode": str,
                "tunnelCode": str,
                "numOfUse": int,
                "addedByUser": bool,
            },
            "returnType": list,
        }

        return self.deeper._api_call(desc)

    def delete_tunnels(self, tunnels: list[str]) -> bool:
        """delete a tunnel

        Args:
            tunnels (list[str]): list of tunnels described by regionCountryTexts like ["DE", "DK"]...

        Returns:
            bool: success
        """
        if isinstance(tunnels, list):
            postData = list([f for f in tunnels if isinstance(f, str)])

        desc = {
            "serviceName": "delete_tunnels",
            "refUrl": self.refUrl,
            "apiRoute": "smartRoute/deleteTunnel",
            "requestMethod": "POST",
            "responseType": dict,
            "responseContent": {"success": bool},
            "returnType": bool,
            "setValue": True,
            "postData": postData,
        }

        return self.deeper._api_call(desc)

    def add_tunnel(self, region: str, country: str) -> dict:

        if not all([isinstance(region, str), isinstance(country, str)]):
            raise TypeError("region and country must be of type string")

        postData = {"regionCode": region, "countryCode": country}

        desc = {
            "serviceName": "add_tunnel",
            "refUrl": self.refUrl,
            "apiRoute": "smartRoute/addTunnel",
            "requestMethod": "POST",
            "responseType": dict,
            "responseContent": {"data": dict, "reason": str, "success": bool},
            "returnType": dict,
            "setValue": True,
            "postData": postData,
        }

        return self.deeper._api_call(desc)

    def switch_node(self, tunnelCode: str, currentIp: str) -> dict:

        if not all([isinstance(tunnelCode, str), isinstance(currentIp, str)]):
            raise TypeError("tunnelCode and currentIp must be of type string")

        postData = {"tunnelCode": tunnelCode, "currentIp": currentIp}
        desc = {
            "serviceName": "switch_node",
            "refUrl": self.refUrl,
            "apiRoute": "smartRoute/switchNode",
            "requestMethod": "POST",
            "responseType": dict,
            "responseContent": {
                "activeIp": str,
                "activeNum": int,
                "success": bool,
            },
            "returnType": dict,
            "setValue": True,
            "postData": postData,
        }

        return self.deeper._api_call(desc)

    def refresh_tunnel(self, tunnelCode: str) -> bool:

        postData = {"tunnelCode": tunnelCode}
        desc = {
            "serviceName": "refresh_tunnel",
            "refUrl": self.refUrl,
            "apiRoute": "smartRoute/refreshTunnel",
            "requestMethod": "POST",
            "responseType": dict,
            "responseContent": {"success": bool},
            "returnType": bool,
            "setValue": True,
            "postData": postData,
        }

        return self.deeper._api_call(desc)



from dataclasses import dataclass
from pydpn._api.api_base import ApiBase
from pydpn._api.mode import DpnMode


@dataclass
class SystemInfo:
    pass


@dataclass
class RealtimeTraffic:
    pass


@dataclass
class WorldmapData:
    pass


class Dashboard(ApiBase):
    @property
    def dpnMode(self) -> int:
        """get the dpn mode

        Returns:
            DpnMode: dpn Mode
        """
        desc = {
            "serviceName": "dpnMode",
            "refUrl": self.refUrl,
            "apiRoute": "smartRoute/getDpnMode",
            "requestMethod": "GET",
            "responseType": dict,
            "responseContent": {"dpnMode": str, "tunnelCode": str},
            "responseAlias": {},
            "returnType": DpnMode,
        }

        return self.deeper._api_call(desc)

    @property
    def memory(self) -> int:
        """get memory used in %

        Returns:
            int: memory used
        """
        desc = {
            "serviceName": "memory",
            "refUrl": self.refUrl,
            "apiRoute": "liquid",
            "requestMethod": "GET",
            "responseType": dict,
            "responseContent": {"scale": int},
            "responseAlias": {"scale": "memory"},
            "returnType": int,
        }

        return self.deeper._api_call(desc)

    @property
    def cpu(self) -> int:
        """get cpu load in %

        Returns:
            int: cpu load
        """
        desc = {
            "serviceName": "cpu",
            "refUrl": self.refUrl,
            "apiRoute": "speedometer",
            "requestMethod": "GET",
            "responseType": dict,
            "responseContent": {"guage": int},
            "responseAlias": {"guage": "cpu"},
            "returnType": int,
        }

        return self.deeper._api_call(desc)

    @property
    def systemInfo(self) -> SystemInfo:
        """get system information

        Returns:
            SystemInfo:
                attrs:
                    - systemUpTime
                    - totalUploads
                    - totalDownloads

        """

        desc = {
            "serviceName": "systemInfo",
            "refUrl": self.refUrl,
            "apiRoute": "info",
            "requestMethod": "GET",
            "responseType": list,
            "responseContent": [
                ("systemUpTime", str),
                ("totalUploads", str),
                ("totalDownloads", str),
            ],
            "returnType": SystemInfo,
        }

        return self.deeper._api_call(desc)

    @property
    def realtimeTraffic(self) -> RealtimeTraffic:
        """get realtime traffic information

        Returns:
            RealtimeTraffic: _description_
        """
        desc = {
            "serviceName": "realtimeTraffic",
            "refUrl": self.refUrl,
            "apiRoute": "dynamic-data",
            "requestMethod": "GET",
            "responseType": dict,
            "responseContent": {
                "downLoadData": list,
                "upLoadData": list,
                "xData": list,
            },
            "returnType": RealtimeTraffic,
        }

        return self.deeper._api_call(desc)

    @property
    def worldmapData(self) -> WorldmapData:
        """get world map information

        Returns:
            WorldmapData: _description_
        """
        desc = {
            "serviceName": "worldmapData",
            "refUrl": self.refUrl,
            "apiRoute": "worldmap-data",
            "requestMethod": "GET",
            "responseType": dict,
            "responseContent": {
                "self": dict,
                "peerList": list,
            },
            "returnType": WorldmapData,
        }
        return self.deeper._api_call(desc)

    @property
    def inform(self) -> list:
        """get world map information

        Returns:
            WorldmapData: _description_
        """
        desc = {
            "serviceName": "inform",
            "refUrl": self.refUrl,
            "apiRoute": "inform",
            "requestMethod": "GET",
            "responseType": list,
            "responseContent": str,
            "returnType": list,
        }
        return self.deeper._api_call(desc)


# class SystemInfo:
#     def __init__(self, inList: list) -> None:

#         if isinstance(inList, list):
#             dlen = len(inList)
#             if dlen < 3:
#                 inList.extend([None] * dlen - 3)
#         else:
#             inList = [None, None, None]

#         self.systemUp = inList[0]
#         self.totalUploads = inList[1]
#         self.totalDownloads = inList[2]

#     def __dict__(self):
#         return {
#             "systemUp": self.systemUp,
#             "totalUploads": self.totalUploads,
#             "totalDownloads": self.totalDownloads,
#         }

#     def __repr__(self) -> str:
#         return json.dumps(self.__dict__())


# class Dashboard(OptionsBase):
#     def __init__(self, deeper) -> None:
#         self.deeper = deeper
#         self.apiRoutes = [
#             "info",
#             "inform",
#             "speedometer",
#             "liquid",
#             "smartRoute/getDpnMode",
#             "worldmap-data",
#             "dynamic-data",
#         ]

#         self._refUrl = self.deeper.hostIp

#         super().__init__()

#     def __repr__(self) -> str:

#         retDict = self.get_values()
#         for key in retDict.keys():
#             if hasattr(retDict[key], "__dict__"):
#                 retDict[key] = retDict[key].__dict__()

#         return json.dumps(retDict)

#     def get_values(self):
#         self.update()

#         _dpnMode = self._load_deeper_mode()
#         dpnMode = pick(_dpnMode, "dpnMode")
#         tunnelCode = pick(_dpnMode, "tunnelCode")

#         return {
#             "memoryUsed": self._load_memory_used(),
#             "cpuLoad": self._load_cpu_load(),
#             "dpnMode": dpnMode,
#             "tunnelCode": tunnelCode,
#             "systemInfo": self._load_system_info(),
#         }

#     def _load_memory_used(self):
#         self.update("liquid")
#         data = self.get_api_data("liquid")
#         if data:
#             if "scale" in data.keys():
#                 return int(data["scale"])

#     @property
#     def memoryUsed(self):
#         return self._load_memory_used()

#     def _load_cpu_load(self):
#         data = self.get_api_data("speedometer")
#         if data:
#             if "guage" in data.keys():
#                 return int(data["guage"])

#     @property
#     def cpuLoad(self):
#         return self._load_cpu_load()

#     def _load_deeper_mode(self, key: str = ""):
#         data = self.get_api_data("smartRoute/getDpnMode")
#         if data:
#             if key:
#                 if key in data.keys():
#                     return data[key]
#             else:
#                 return data

#     @property
#     def dpnMode(self):
#         return self._load_deeper_mode("dpnMode")

#     @property
#     def tunnelCode(self):
#         return self._load_deeper_mode("tunnelCode")

#     def _load_system_info(self) -> SystemInfo:
#         return SystemInfo(self.get_api_data("info"))

#     @property
#     def systemInfo(self):
#         return self._load_system_info(self)



from dataclasses import dataclass
from pydpn._api.api_base import ApiBase


@dataclass
class Notifications:
    pass


class Log(ApiBase):
    @property
    def notifications(self) -> Notifications:
        """get the history of all notifications
        Returns:
            Notifications:
        """
        desc = {
            "serviceName": "notifications",
            "refUrl": self.refUrl,
            "apiRoute": "notifications",
            "requestMethod": "GET",
            "responseType": dict,
            "responseContent": {"CP": list, "DP": list, "MP": list},
            "returnType": Notifications,
        }

        return self.deeper._api_call(desc)



from .dashboard import Dashboard
from .system import System
from .traffic import Traffic
from .log import Log
from .mode import Mode
from .mode import DpnMode
from .tunnel import Tunnel
from .routing import Routing
from .web_filter import WebFilter
from .ssl_filter import SslFilter
from .tcp_access_control import TcpAccessControl
from .sharing_config import SharingConfig
from .sharing_filter import SharingFilter
from .sharing_log import SharingLog
from .sharing_traffic import SharingTraffic
from .password import Password
from .reboot import Reboot


__all__ = [
    "Dashboard",
    "System",
    "Traffic",
    "Log",
    "Mode",
    "DpnMode",
    "Tunnel",
    "Routing",
    "WebFilter",
    "SslFilter",
    "TcpAccessControl",
    "SharingConfig",
    "SharingFilter",
    "SharingLog",
    "SharingTraffic",
    "Password",
    "Reboot",
]



from typing import Any, Union
import base64
import json

import requests
from requests import Response
from bs4 import BeautifulSoup
from Crypto.PublicKey.RSA import import_key
from Crypto.Cipher import PKCS1_OAEP
from pydpn._utils.api_service import ApiService
from pydpn._api import *


class DeeperNetwork:
    def __init__(
        self,
        username,
        password,
        hostIp: str = "http://34.34.34.34",
        proxy: Union[str, dict] = "",
    ) -> None:

        self.username: str = username
        self.password: str = password
        self.hostIp: str = hostIp

        # check for proxy settings
        if proxy:
            # convert to requests proxy schema
            if isinstance(proxy, str):
                self.proxy: dict = {
                    "http": f"http://{proxy}",
                    "https": f"https://{proxy}",
                }
            elif isinstance(self.proxy, dict):
                self.proxy: dict = proxy
        else:
            self.proxy: dict = None

        self.login()
        self.lastPostResp = {}

    def __enter__(self):
        return self

    def __exit__(self, exception_type, exception_value, traceback):
        self.logout()

    def login(self, username: str = "", password: str = "") -> bool:
        """login to deeper device

        Args:
            username (str, optional): username. Defaults to "".
            password (str, optional): password. Defaults to "".

        Raises:
            ValueError: _description_
            ValueError: _description_
            ValueError: _description_
            ValueError: _description_

        Returns:
            bool: login success
        """

        # option for changing username and password after init
        if username:
            self.username = username

        if password:
            self.password = password

        # create requests session
        self.session = requests.Session()
        self.session.proxies.update(self.proxy)

        # load a public key from deeper device
        pubKey = self._load_public_key()
        if pubKey is None:
            raise ValueError("No Public Key found in js scrips provided by host")

        # encrypt the passwort with public key
        encPassword = PKCS1_OAEP.new(import_key(pubKey, passphrase=None)).encrypt(
            bytes(self.password, encoding="UTF-8")
        )
        data = {
            "username": self.username,
            "password": base64.b64encode(encPassword).decode("utf-8"),
        }
        url = f"{self.hostIp}/api/admin/login"

        # send login with post request to host
        # self.session = requests.Session()
        resp = self.session.post(url, data=data, proxies=self.proxy)

        # analyse host response
        if "application/json" in resp.headers["Content-Type"]:
            respData = json.loads(resp.text)
            respDataKeys = respData.keys()
            if "success" in respDataKeys:
                if respData["success"]:
                    if "token" in respDataKeys:
                        self.loginToken = respData["token"]
                        self.session.headers.update(
                            {"Authorization": respData["token"]}
                        )
                        return True

                    else:
                        raise ValueError("Token not transfered from host")

            elif "wrongPassword" in respDataKeys:
                raise ValueError("Password incorrect")

            elif "wrongUsername" in respDataKeys:
                raise ValueError("Username incorrect")

    def logout(self):
        url = f"{self.hostIp}/system?sessionError=sessionTimeout"
        self.session.headers.update({"Connection": "close"})
        resp = self.session.get(url)
        resp.close()

    @property
    def sharing_config(self):
        return SharingConfig(self)

    @property
    def dashboard(self):
        return Dashboard(self)

    @property
    def system(self):
        return System(self)

    @property
    def mode(self):
        return Mode(self)

    @property
    def tunnel(self):
        return Tunnel(self)

    @property
    def routing(self):
        return Routing(self)

    def reset(self):

        self.__init__(self.username, self.password)

    def quit(self):
        print("close")

    def _load_public_key(self) -> str:
        """method is loading all java script files provided by host
           and is searching for a public key (RSA)

        Returns:
            str: public key
        """

        soup = BeautifulSoup(self.session.get(self.hostIp).text, features="lxml")
        beginPublicKey = "-----BEGIN PUBLIC KEY-----"
        endPublicKey = "-----END PUBLIC KEY-----"
        for script in soup.find_all("script"):
            if script.attrs:
                if "src" in script.attrs.keys():
                    jsUrl = self.hostIp + script.attrs["src"]
                    resp = self.session.get(jsUrl, proxies=self.proxy)
                    if all(
                        [
                            beginPublicKey in resp.text,
                            endPublicKey in resp.text,
                        ]
                    ):
                        pubKeyText = resp.text.split(beginPublicKey)[1].split(
                            endPublicKey
                        )[0]
                        return (
                            beginPublicKey
                            + pubKeyText.replace("\\n", "\n")
                            + endPublicKey
                        )

    def _api_get(self, refUrl: str, apiRoute: str) -> Response:
        self.session.headers.update({"Referer": refUrl})
        url = f"{self.hostIp}/api/{apiRoute}"
        return self.session.get(url)

    def _api_post(self, refUrl: str, apiRoute: str, data) -> Response:
        self.session.headers.update({"Referer": refUrl})
        url = f"{self.hostIp}/api/{apiRoute}"
        resp = self.session.post(url, json=data)
        return resp

    def _api_call(self, service: dict) -> Any:

        service = ApiService(**service)
        if service.requestMethod == "GET":
            resp = self._api_get(service.refUrl, service.apiRoute)

        elif service.requestMethod == "POST":
            resp = self._api_post(service.refUrl, service.apiRoute, service.postData)

        if resp.status_code == 200:
            # if service.setValue:
            #     return json.loads(resp.text)
            # else:
            data = json.loads(resp.text)

            if service.responseType is dict:
                for key in data:
                    data[key] = service.responseContent[key](data[key])
                    if service.returnType in [int, float, str]:
                        return data[key]

                if service.returnType not in [
                    dict,
                    list,
                    float,
                    int,
                    str,
                    bool,
                ]:
                    resp = service.returnType()
                    for key in data:
                        setattr(resp, key, data[key])

                    return resp

                if service.returnType is bool:
                    for key in data:
                        return bool(data[key])

            if service.responseType is dict:
                for key in service.responseContent:
                    if key in data:

                        retVal = data[key]

                    else:
                        retVal = None

                    if key in service.responseAlias:
                        # setattr(self, service.responseAlias[key], retVal)
                        pass
                    else:
                        setattr(self, key, retVal)

                    retData = {}
                    for key in service.responseAlias:
                        retData.update({})

            elif service.responseType is list:
                if service.returnType not in [dict, list, float, int, str]:
                    resp = service.returnType()
                    for el, desc in zip(data, service.responseContent):
                        setattr(resp, desc[0], desc[1](el))

                    return resp

        return data



from typing import Literal
from dataclasses import dataclass
from pydpn._api.api_base import ApiBase
from pydpn._utils.misc import valid_ip

from re import match


class Routing(ApiBase):
    def get_routings(
        self,
        routeType: Literal["ip", "domain"],
        mode: Literal["smart", "direct"],
        page: int = 1,
        size: int = 1000,
    ) -> list:

        if mode == "smart":
            smartDirectSelect = "getRoutingWhitelist"

        elif mode == "direct":
            smartDirectSelect = "getRoutingBlacklist"

        desc = {
            "serviceName": "get_routings",
            "refUrl": f"{self.deeper.hostIp}/{routeType}Config",
            "apiRoute": f"smartRoute/{smartDirectSelect}/{routeType}?pageNo={page}&pageSize={size}",
            "requestMethod": "GET",
            "responseType": dict,
            "responseContent": {"total": int, "list": list},
            "returnType": list,
        }

        return self.deeper._api_call(desc)

    def add_route(self, domainIp, tunnelCode: str = ""):

        if match("^((25[0-5]|(2[0-4]|1\d|[1-9]|)\d)\.?\b){4}$", domainIp):
            domainIpSelect = "ip"

        else:
            domainIpSelect = "domain"

        if tunnelCode:
            smartDirectSelect = "addToWhitelist"

        else:
            smartDirectSelect = "addToBlacklist"

        postData = {domainIpSelect: domainIp}
        desc = {
            "serviceName": "switch_node",
            "refUrl": f"{self.deeper.hostIp}/{domainIpSelect}Config",
            "apiRoute": f"smartRoute/{smartDirectSelect}/{domainIpSelect}",
            "requestMethod": "POST",
            "responseType": dict,
            "responseContent": {
                "activeIp": str,
                "activeNum": int,
                "success": bool,
            },
            "returnType": dict,
            "setValue": True,
            "postData": postData,
        }

        return self.deeper._api_call(desc)

    def delete_route(
        self, route: list[str], mode: Literal["smart", "direct"]
    ) -> bool:

        evalInput = [valid_ip(f) for f in route]
        if all(evalInput):
            domainIpSelect = "ip"

        elif any(evalInput):
            raise ValueError("Input must be all ip or all domain not mixed")

        else:
            domainIpSelect = "domain"

        if mode == "smart":
            smartDirectSelect = "deleteFromWhitelist"

        elif mode == "direct":
            smartDirectSelect = "deleteFromBlacklist"

        postData = route
        desc = {
            "serviceName": "switch_node",
            "refUrl": f"{self.deeper.hostIp}/{domainIpSelect}Config",
            "apiRoute": f"smartRoute/{smartDirectSelect}/{domainIpSelect}",
            "requestMethod": "POST",
            "responseType": dict,
            "responseContent": {
                "success": bool,
            },
            "returnType": dict,
            "setValue": True,
            "postData": postData,
        }

        return self.deeper._api_call(desc)



import os, json

def check_config()->dict:
    usrdir = os.path.expanduser("~")
    configFile = os.path.join(usrdir, r".dpn/config.json")
    if os.path.isfile(configFile):
        return json.load(open(configFile, "r"))

    else:

        print(f'There is no "config.json" file at "{os.path.split(configFile)[0]}"')
        return None


from dataclasses import dataclass
from pydpn._api.api_base import ApiBase


@dataclass
class SessionSpeed:
    pass


@dataclass
class TotalTraffic:
    pass


@dataclass
class TrafficStatistics:
    pass


class Traffic(ApiBase):
    @property
    def sessionSpeed(self) -> SessionSpeed:
        """get the history of traffic speed
        Returns:
            SessionSpeed:
        """
        desc = {
            "serviceName": "sessionSpeed",
            "refUrl": self.refUrl,
            "apiRoute": "traffic/session-speed",
            "requestMethod": "GET",
            "responseType": dict,
            "responseContent": {
                "timeline": list,
                "traffic": list,
            },
            "returnType": SessionSpeed,
        }

        return self.deeper._api_call(desc)

    @property
    def totalTraffic(self) -> TotalTraffic:
        """get the total traffic
        Returns:
            TotalTraffic:
        """
        desc = {
            "serviceName": "totalTraffic",
            "refUrl": self.refUrl,
            "apiRoute": "traffic/total-traffic",
            "requestMethod": "GET",
            "responseType": dict,
            "responseContent": {
                "clientTraffic": dict,
                "localTraffic": dict,
                "serverTraffic": dict,
            },
            "returnType": TotalTraffic,
        }

        return self.deeper._api_call(desc)

    @property
    def totalStatistics(self) -> TrafficStatistics:
        """get the history of total traffic
        Returns:
            TrafficStatistics:
        """
        desc = {
            "serviceName": "totalStatistics",
            "refUrl": self.refUrl,
            "apiRoute": "traffic/total-statistics",
            "requestMethod": "GET",
            "responseType": dict,
            "responseContent": {
                "bandwidthAppRatio": dict,
                "sessionAppRatio": dict,
            },
            "returnType": TrafficStatistics,
        }

        return self.deeper._api_call(desc)

    @property
    def selfStatistics(self) -> TrafficStatistics:
        """get the history of self-traffic
        Returns:
            TrafficStatistics:
        """
        desc = {
            "serviceName": "selfStatistics",
            "refUrl": self.refUrl,
            "apiRoute": "traffic/self-statistics",
            "requestMethod": "GET",
            "responseType": dict,
            "responseContent": {
                "bandwidthAppRatio": dict,
                "sessionAppRatio": dict,
            },
            "returnType": TrafficStatistics,
        }

        return self.deeper._api_call(desc)

    @property
    def shareStatistics(self) -> TrafficStatistics:
        """get the history of self-traffic
        Returns:
            TrafficStatistics:
        """
        desc = {
            "serviceName": "shareStatistics",
            "refUrl": self.refUrl,
            "apiRoute": "traffic/share-statistics",
            "requestMethod": "GET",
            "responseType": dict,
            "responseContent": {
                "bandwidthAppRatio": dict,
                "sessionAppRatio": dict,
            },
            "returnType": TrafficStatistics,
        }

        return self.deeper._api_call(desc)




import typer
from pydpn import DeeperNetwork
from pydpn._utils.cli_tools import check_config
app = typer.Typer()

config = check_config()

@app.command("memory")
def memory():
    """used memory from dpn device

    Args:
       
    """
    print("memory")
    if config:

        d = DeeperNetwork(**config).dashboard
        print(d.memory)


@app.command("cpu")
def cpu():
    """used memory from dpn device

    Args:
       
    """
    print("cpu")
    if config:
        d = DeeperNetwork(**config).dashboard
        print(d.cpu)


def main():
    """main"""
    app()



from dataclasses import dataclass


@dataclass
class HardwareInfo:
    pass


@dataclass
class SoftwareInfo:
    pass


@dataclass
class NetworkAddress:
    pass


@dataclass
class SessionInfo:
    pass


@dataclass
class LatestSoftwareVersion:
    pass


@dataclass
class SystemUsageHistory:
    pass


class System:
    def __init__(self, deeper) -> None:

        self.deeper = deeper
        self.refUrl = f"{self.deeper.hostIp}/system"

    @property
    def hardwareInfo(self) -> HardwareInfo:
        """get hardware information

        Returns:
            HardwareInfo:
        """
        desc = {
            "serviceName": "hardwareInfo",
            "refUrl": self.refUrl,
            "apiRoute": "system-info/hardware-info",
            "requestMethod": "GET",
            "responseType": dict,
            "responseContent": {
                "SN": str,
                "cpuCount": int,
                "cpuModel": str,
                "deviceId": str,
                "totalMem": int,
            },
            "responseAlias": {},
            "returnType": HardwareInfo,
        }

        return self.deeper._api_call(desc)

    @property
    def softwareInfo(self) -> SoftwareInfo:
        """get hardware information

        Returns:
            HardwareInfo:
        """
        desc = {
            "serviceName": "hardwareInfo",
            "refUrl": self.refUrl,
            "apiRoute": "system-info/software-info",
            "requestMethod": "GET",
            "responseType": dict,
            "responseContent": {
                "appSigVersion": str,
                "softwareVersion": str,
                "urlSigVersion": str,
            },
            "responseAlias": {},
            "returnType": SoftwareInfo,
        }

        return self.deeper._api_call(desc)

    @property
    def networkAddress(self) -> NetworkAddress:
        """get hardware information

        Returns:
            HardwareInfo:
        """
        desc = {
            "serviceName": "networkAddress",
            "refUrl": self.refUrl,
            "apiRoute": "system-info/network-address",
            "requestMethod": "GET",
            "responseType": dict,
            "responseContent": {
                "gatewayMac": str,
                "ip": str,
                "routerMac": str,
            },
            "responseAlias": {},
            "returnType": NetworkAddress,
        }

        return self.deeper._api_call(desc)

    @property
    def sessionInfo(self) -> SessionInfo:
        """get hardware information

        Returns:
            HardwareInfo:
        """
        desc = {
            "serviceName": "sessionInfo",
            "refUrl": self.refUrl,
            "apiRoute": "system-info/session-info",
            "requestMethod": "GET",
            "responseType": dict,
            "responseContent": {
                "currSessionNum": int,
                "icmpSessionNum": int,
                "maxSessionNum": int,
                "tcpSessionNum": int,
                "tunnelSessionNum": int,
                "udpSessionNum": int,
            },
            "responseAlias": {},
            "returnType": SessionInfo,
        }

        return self.deeper._api_call(desc)

    @property
    def autoUpdate(self) -> bool:
        """get the status of auto update function

        Returns:
            SystemUsageHistory:
        """
        desc = {
            "serviceName": "systemUsageHistory",
            "refUrl": self.refUrl,
            "apiRoute": "system-info/get-autoupdate",
            "requestMethod": "GET",
            "responseType": dict,
            "responseContent": {
                "autoUpdate": bool,
            },
            "responseAlias": {},
            "returnType": bool,
        }

        return self.deeper._api_call(desc)

    @autoUpdate.setter
    def autoUpdate(self, value: bool) -> dict:
        """set the status of auto update function

        Returns:
            dict:
        """
        if not isinstance(value, bool):
            raise ValueError("autoUpdate must be True or False")

        desc = {
            "serviceName": "set_autoUpdate",
            "refUrl": self.refUrl,
            "apiRoute": f"system-info/set-autoupdate?autoUpdate={int(value)}",
            "requestMethod": "GET",
            "responseType": dict,
            "responseContent": {"code": bool, "desc": str},
            "responseAlias": {},
            "returnType": dict,
            "setValue": True,
        }

        self.deeper.lastUpdate = self.deeper._api_call(desc)

    @property
    def latestSoftwareVersion(self) -> LatestSoftwareVersion:
        """get the current installed and latest avalible software version
        Returns:
            LatestSoftwareVersion:
        """
        desc = {
            "serviceName": "latestSoftwareVersion",
            "refUrl": self.refUrl,
            "apiRoute": "system-info/get-latestversion",
            "requestMethod": "GET",
            "responseType": dict,
            "responseContent": {
                "currentVersion": str,
                "latestVersion": str,
            },
            "responseAlias": {},
            "returnType": LatestSoftwareVersion,
        }

        return self.deeper._api_call(desc)

    @property
    def systemUsageHistory(self) -> SystemUsageHistory:
        """get 24 hour history of cpu and memory usage

        Returns:
            SystemUsageHistory:
        """
        desc = {
            "serviceName": "systemUsageHistory",
            "refUrl": self.refUrl,
            "apiRoute": "system-info/usage",
            "requestMethod": "GET",
            "responseType": dict,
            "responseContent": {
                "cpuUsageData": list,
                "memUsageData": list,
                "xData": list,
            },
            "responseAlias": {},
            "returnType": SystemUsageHistory,
        }

        return self.deeper._api_call(desc)



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['pydpn', 'pydpn._api', 'pydpn._const', 'pydpn._utils']

package_data = \
{'': ['*']}

install_requires = \
['beautifulsoup4>=4.11.1,<5.0.0',
 'lxml>=4.9.2,<5.0.0',
 'pycryptodome>=3.16.0,<4.0.0',
 'requests>=2.28.1,<3.0.0',
 'typer>=0.7.0,<0.8.0']

entry_points = \
{'console_scripts': ['dpn = pydpn.cli:main']}

setup_kwargs = {
    'name': 'pydpn',
    'version': '0.1.7',
    'description': 'python package for configuring a deeperNetwork device',
    'long_description': '# pydpn\n python package for configuring a deeperNetwork device\n',
    'author': 'Your Name',
    'author_email': 'you@example.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



from deepernetwork import DeeperNetwork


deeper = DeeperNetwork(
    username="admin", password="", hostIp="http://34.34.34.34", proxy=""
)

newsUpdates = deeper.dashboard.inform

aktMode = deeper.dashboard.dpnMode
if aktMode != "full":

    response = deeper.mode.set_dpnMode(
        {
            "dpnMode": "full",
            "tunnelCode": "EUE",
        }
    )

    if response is True:
        print("active dpn mode: full")
